/***********************************************
 * @brief Function to start advertising.					*
 *																							*
 * 														*
 ************************************************/
static void advertising_start(void)
{
	printf("[MAIN : BLE] Advertising started.\r\n");
	
	uint32_t err_code;
	err_code = ble_advertising_start(BLE_ADV_MODE_FAST);
  APP_ERROR_CHECK(err_code);
	
}

/******************************************************************************************
 * @brief Function for the SoftDevice initialization.																			*
 *																																												*
 * @details This function initializes the SoftDevice and the BLE event interrupt.					*
 ******************************************************************************************/
static void ble_stack_init(void)
{
    uint32_t err_code;
    
    //nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;//old one		
	
    nrf_clock_lf_cfg_t clock_lf_cfg = { .source = NRF_CLOCK_LF_SRC_RC,		\
																				.rc_ctiv = 4,	\
																				.rc_temp_ctiv = 1};
	//nrf_clock_lf_cfg_t clock_lf_cfg = {.source = NRF_CLOCK_LF_SRC_SYNTH,	\
																		.rc_ctiv = 0,	\
																		.rc_temp_ctiv = 0};
    
    // Initialize SoftDevice.
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);
    
    ble_enable_params_t ble_enable_params;
    err_code = softdevice_enable_get_default_config(CENTRAL_LINK_COUNT,
                                                    PERIPHERAL_LINK_COUNT,
                                                    &ble_enable_params);
    APP_ERROR_CHECK(err_code);
        
    //Check the ram settings against the used number of links
    CHECK_RAM_START_ADDR(CENTRAL_LINK_COUNT,PERIPHERAL_LINK_COUNT);
    // Enable BLE stack.
    err_code = softdevice_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);
    
    // Subscribe for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
}

/********************************************************************
 * @brief Function for initializing the Advertising functionality.	*
 ********************************************************************/
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    ble_advdata_t scanrsp;

    // Build advertising data struct to pass into @ref ble_advertising_init.
    memset(&advdata, 0, sizeof(advdata));
    advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance = false;
    advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

    memset(&scanrsp, 0, sizeof(scanrsp));
    scanrsp.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    scanrsp.uuids_complete.p_uuids  = m_adv_uuids;

    ble_adv_modes_config_t options = {0};
    options.ble_adv_fast_enabled  = BLE_ADV_FAST_ENABLED;
    options.ble_adv_fast_interval = APP_ADV_INTERVAL;
    options.ble_adv_fast_timeout  = APP_ADV_TIMEOUT_IN_SECONDS;

    err_code = ble_advertising_init(&advdata, &scanrsp, &options, on_adv_evt, NULL);
    APP_ERROR_CHECK(err_code);
}


/****************************************
 *@brief Application main function.			*
 ****************************************/
int main(void)
{
    uint32_t err_code;
    bool erase_bonds;

    // Initialize.
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
    uart_init();			//Function for initializing the UART module
    buttons_leds_init(&erase_bonds);		//Function for initializing buttons and leds.	
	ble_stack_init();		//Function for the SoftDevice initialization.
    gap_params_init();		//Function for the GAP initialization
    services_init();		//Function for initializing services that will be used by the application
    advertising_init();		//Function for initializing the Advertising functionality
    conn_params_init();		//Function for initializing the Connection Parameters module
	spi_init();				//Function for configurating the pins for SPI
	
	app_timer_create(&m_counter_timer_id, APP_TIMER_MODE_REPEATED, counter_timer_handler);

	//starting advertising
	advertising_start();
	
	// Enter main loop.
    for (;;)
    {
		power_manage();		//Function for placing the application in low power state while waiting for events
    }
}